#include <stdio.h>
#include <stdlib.h>
#include "list.h"

/**
 * Name: init_list
 * Input parameters: list * int
 * Return Type:void
 * Objective: initialize a list 
*/
List * init_list() {
  int i = 0;
  List *l = (List*)malloc(sizeof(List));
  
  if(!l)
    exit (-1);
  
  l->head = NULL;
  l->tail = NULL;
  l->size = 0;
   
  return l;
}



/**
 * Name: insert_node
 * Input parameters: nodes*, int
 * Return Type: void
 * Objective: insert node in the list "l" at the tail of the list. 
*/
void insert_node(List *l, int val) {
  Node *cur; 

  Node *new_node = (Node*)malloc(sizeof(Node));
  if(!new_node)
    exit(-1);
  
  new_node->data = val;
  new_node->next = NULL;
  
  if (l->head == NULL){
    new_node->next = NULL;
    l->head = new_node;
    l->tail = new_node;
  }
  else{
    l->tail->next = new_node;
    l->tail= new_node;
  }

  ++l->size;

}


/**
 * Name: delete_list
 * Input parameter: list*
 * Return type: void
 * Objective: delete contents of list "l". 
*/
void delete_list(List *l){
  int i = 0;
  Node* d_node;
    
  //empty list
  if (!l->head){
    printf("No list to delete");
    exit(-1);
  }

  for (i = 0; i < l->size ;++i){
        
    d_node = l->head;
    l->head = l->head->next;
    free(d_node);
    d_node = NULL;
    --l->size;
  }
  
  free(l);
}

/**
 * Name: display_list
 * Input parameters: list*
 * Return type: void
 * Objective: displays the content of the list "l".
*/
void display_list(List *l){
  int i = 0;
  Node *p_node =l->head;
    
  //empty list
  if (!l->head){
    printf("The list is empty");
    exit(-1);
  }

  printf("%d-",p_node->data);
  p_node = l->head->next;
  for(i=1; i < l->size;++i){
    printf("%d-",p_node->data);
    p_node = p_node->next;
  }
  printf("\n");
}


void RemoveDuplicates(List *l) {
  Node* current = l->head;

  if (current == NULL) return; // do nothing if the list is empty
  // Compare current node with next node
  
  while(current->next!=NULL) {
    if (current->data == current->next->data) {
      Node* nextNext = current->next->next;
      free(current->next);
      l->size--;
      current->next = nextNext;
    }
    else {
      current = current->next; // only advance if no deletion
    }
  }
}


void insert_sorted(List *l, int val) {
	Node *pre = NULL, *cur;
	
	Node *new_node = (Node*)malloc(sizeof(Node));
    if(!new_node)
                 exit(-1);
  
   new_node->data = val;
   new_node->next = NULL;
  
	cur = l->head;
	while (cur) {
		if (cur->data > val)
			break;
		
		pre = cur;
		cur = cur->next;
	}
	
	if (pre)
		pre->next = new_node;
		
	new_node->next = cur;
	l->size++;
	
	if (!pre) {
		l->head = new_node;
		l->size++;
	}
}
